import exceptions
import os
import socket
import subprocess
import time

from gtk import gdk

ACTION_HALT = 1
ACTION_REBOOT = 2
ACTION_SLEEP = 4

MESSAGE_SWITCH_USER = "FLEXI_XSERVER"

PRIMARY_SOCKET_PATH = "/var/run/gdm_socket"
SECONDARY_SOCKET_PATH = "/tmp/.gdm_socket"

class GdmException(exceptions.Exception):
	pass

def get_display_num():
	display_name = gdk.display_get_default().get_name()
	i = display_name.find(':')
	if i < 0:
		return 0
	s = display_name[i + 1:].lstrip(':')
	i = s.index('.')
	if i != -1:
		s = s[:i]
	return int(s)

class Protocol:
	def __init__(self):
		self.available_actions = 0
		self.current_actions = 0
		self.last_update = None
		self.hostname = socket.gethostname()
		self.sock = None
		self.auth_cookie = None
		self.connect()

	def send_message(self, message, reset_last_update = True):
		self.connect()
		response = self.__send_message(message)
		self.shutdown()
		if reset_last_update:
			self.last_update = None
		return response

	def connect(self):
		self.open_socket()
		try:
			response = self.__send_message("VERSION")
			if not response.startswith("GDM"):
				raise GdmException( \
						_("Incorrect response to GDM version message"))
			if not self.authenticate():
				raise GdmException( \
						_("Failed to authenticate with GDM server"))
		except:
			self.shutdown()
			raise
	
	def shutdown(self):
		if not self.sock:
			return
		self.auth_cookie = None
		try:
			self.sock.close()
		except:
			pass
		self.sock = None

	def open_socket(self):
		if self.sock:
			return
		try:
			self.sock = socket.socket(socket.AF_UNIX)
			if os.path.exists(PRIMARY_SOCKET_PATH):
				self.sock.connect(PRIMARY_SOCKET_PATH)
			else:
				self.sock.connect(SECONDARY_SOCKET_PATH)
		except Exception, e:
			if self.sock:
				try:
					self.sock.close()
				except:
					pass
				self.sock = None
			raise e
	
	def __send_message(self, message):
		self.sock.sendall(message + '\n')
		response = ""
		while not '\n' in response:
			response += self.sock.recv(256)
		return response.rstrip()

	def authenticate(self):
		if self.auth_cookie:
			if self.__send_message("AUTH_LOCAL " + self.auth_cookie) == "OK":
				return True
			else:
				self.auth_cookie = None
		# python-xlib Xauth doesn't work, so use xauth command
		xauth = subprocess.Popen(['xauth', 'list', os.environ['DISPLAY']],
				stdout = subprocess.PIPE)
		xauth.wait()
		for l in xauth.stdout.readlines():
			a, n, d = l.split()
			if len(d) == 32 and n == "MIT-MAGIC-COOKIE-1":
				try:
					a, n = a.split(':')
					h, a = a.split('/')
					if h == self.hostname and a == 'unix' and \
							self.__send_message("AUTH_LOCAL " + d) == "OK":
						self.auth_cookie = d
						return True
				except:
					continue
		return False

	def update_logout_actions(self):
		t = time.time()
		if self.last_update != None and t < self.last_update + 1:
			return
		response = self.send_message("QUERY_LOGOUT_ACTION", False)
		if not response:
			return
		if not response.startswith("OK "):
			return
		self.current_actions = 0
		self.available_actions = 0
		actions = response[3:].split(';')
		self.last_update= t
		for a in actions:
			if a[-1] == '!':
				current = True
				a = a[:-1]
			else:
				current = False
			if a == "HALT":
				a = ACTION_HALT
			elif a == "REBOOT":
				a = ACTION_REBOOT
			elif a == "SUSPEND":
				a = ACTION_SLEEP
			else:
				a = 0
			if current:
				self.current_actions |= a
			else:
				self.available_actions |= a
	
	def supports_logout_action(self, action):
		self.update_logout_actions()
		return (self.available_actions & action) != 0

	def set_logout_action(self, action):
		if action == ACTION_HALT:
			action = "HALT"
		elif action == ACTION_REBOOT:
			action = "REBOOT"
		elif action == ACTION_SLEEP:
			action = "SUSPEND"
		else:
			return
		response = self.send_message("SET_SAFE_LOGOUT_ACTION " + action)
		return response.startswith("OK")
