import rox
from rox import g
from rox.options import Option

import os, sys, subprocess

import gdm
import sessionbus

rox.setup_app_options('ROX-Session', 'Options', 'rox.sourceforge.net')

halt_command = Option('halt_command', 'halt')
reboot_command = Option('reboot_command', 'reboot')
suspend_command = Option('suspend_command', 'pm-suspend')

rox.app_options.notify()

# Load icons
factory = g.IconFactory()
for name in ['rox-suspend', 'rox-halt']:
	path = os.path.join(rox.app_dir, "images", name + ".png")
	pixbuf = g.gdk.pixbuf_new_from_file(path)
	if not pixbuf:
		print >>sys.stderr, "Can't load stock icon '%s'" % name
	g.stock_add([(name, name, 0, 0, "")])
	factory.add(name, g.IconSet(pixbuf = pixbuf))
factory.add_default()

class OpButton(g.Button):
	def __init__(self, text, stock, message = None, dialog = None,
			response = g.RESPONSE_ACCEPT):
		g.Button.__init__(self)

		self.dialog = dialog

		hbox = g.HBox(False, 0)
		self.add(hbox)

		image = g.Image()
		image.set_from_stock(stock, g.ICON_SIZE_BUTTON)
		hbox.pack_start(image, False, True, 4)

		label = g.Label('')
		label.set_text_with_mnemonic(text)
		label.set_alignment(0, 0.5)
		hbox.pack_start(label, True, True, 0)

		self.message = message
		self.response = response
		self.connect('clicked', self.clicked_cb)
	
		self.unset_flags(g.CAN_FOCUS)

	def clicked_cb(self, widget):
		if self.message:
			print >>sys.stderr, self.message
		if self.dialog:
			self.dialog.response(self.response)
		self.invoke()
	
	def invoke(self):
		pass


class CommandOpButton(OpButton):
	def __init__(self, text, stock, command, message = None, dialog = None):
		if command:
			self.command = command.strip()
		else:
			self.command = None
		OpButton.__init__(self, text, stock, message, dialog)
		if not self.command:
			self.set_sensitive(False)

	def invoke(self):
		if self.command:
			subprocess.call(self.command, shell = True)


class GdmOpButton(OpButton):
	""" gdm_command may either be a string or
	one of the gdm.ACTION int constants """
	def __init__(self, text, stock, gdmctl, gdm_command,
			message = None, dialog = None):
		# Only want base class to close dialog for Switch User, otherwise
		# we close it with response indicating logout
		self.real_dialog = dialog
		if isinstance(gdm_command, int):
			dialog = None
		OpButton.__init__(self, text, stock, message, dialog)
		self.gdmctl = gdmctl
		self.gdm_command = gdm_command

	def invoke(self):
		if isinstance(self.gdm_command, int):
			self.gdmctl.set_logout_action(self.gdm_command)
			self.real_dialog.response(g.RESPONSE_YES)
		else:
			self.gdmctl.send_message(self.gdm_command)


class SuspendButton(OpButton):
	def __init__(self, text, stock, power_manager,
			message = None, dialog = None):
		OpButton.__init__(self, text, stock, message, dialog)
		self.power_manager = power_manager

	def invoke(self):
		self.power_manager.Suspend()


def op_button(text, stock, command, message = None,
		gdmctl = None, gdm_command = None, dialog = None):
	if gdmctl and \
			(not isinstance(gdm_command, int) or \
				gdmctl.supports_logout_action(gdm_command)):
		return GdmOpButton(text, stock, gdmctl, gdm_command, message, dialog)
	else:
		return CommandOpButton(text, stock, command, message, dialog)


def suspend_button(text, stock, command, dialog):
	try:
		bus = sessionbus.SessionBus()
		power_manager = sessionbus.Interface( \
				bus.get_object("org.freedesktop.PowerManagement",
						"/org/freedesktop/PowerManagement",
						"org.freedesktop.PowerManagement"),
				"org.freedesktop.PowerManagement")
		if not power_manager.CanSuspend():
			power_manager = None
	except:
		power_manager = None
	if power_manager:
		return SuspendButton(text, stock, power_manager,
				_("Attempting to suspend the system using Power Manager..."),
				dialog)
	else:
		return CommandOpButton(text, stock, command,
				_("Attempting to suspend the system with user command..."),
				dialog)


class LogoutBox(rox.Dialog):
	def __init__(self):
		rox.Dialog.__init__(self)
		# TODO: Check if there's already a logout box...

		try:
			self.gdmctl = gdm.Protocol()
		except:
			self.gdmctl = None
	
		self.set_has_separator(False)
		vbox = self.vbox

		hbox = g.HButtonBox()
		hbox.set_border_width(2)
		hbox.set_layout(g.BUTTONBOX_END)
		vbox.pack_start(hbox, False, True, 0)

		button = op_button(_('_Halt'), 'rox-halt',
			halt_command.value,
			_('Attempting to halt the system...'),
			self.gdmctl, gdm.ACTION_HALT, self)
		hbox.pack_end(button, False, True, 0)

		button = op_button(_('_Reboot'), g.STOCK_REFRESH,
			reboot_command.value,
			_('Attempting to restart the system...'),
			self.gdmctl, gdm.ACTION_REBOOT, self)
		hbox.pack_end(button, False, True, 0)
		
		button = suspend_button(_('_Sleep'), 'rox-suspend',
			suspend_command.value, self)
		hbox.pack_end(button, False, True, 0)

		vbox.pack_start(g.Label(
			_('Really logout?\n(unsaved data will be lost)')),
			True, True, 16)
		hbox = g.HBox()
		button = OpButton(_("Session Settings"),
				g.STOCK_PREFERENCES, None, self, 1)
		hbox.pack_end(button, False, False, 8)
		vbox.pack_start(hbox, False)

		vbox.show_all()

		self.set_title(_('ROX-Session'))

		self.set_position(g.WIN_POS_CENTER)

		self.add_button(g.STOCK_CANCEL, g.RESPONSE_CANCEL)

		button = op_button(_('_Switch User'), g.STOCK_JUMP_TO,
			None,
			_('Attempting to switch user...'),
			self.gdmctl, gdm.MESSAGE_SWITCH_USER)
		button.show_all()
		self.add_action_widget(button, g.RESPONSE_ACCEPT)

		button = rox.ButtonMixed(g.STOCK_QUIT, _("Logout"))
		button.set_flags(g.CAN_DEFAULT)
		button.show()
		self.add_action_widget(button, g.RESPONSE_YES)

		self.set_default_response(g.RESPONSE_YES)


def show_logout_box(session_control):
	box = LogoutBox()
	try:
		resp = box.run()
	except:
		box.destroy()
		raise
	if resp == g.RESPONSE_YES:
		session_control.LogoutWithoutConfirm()
	elif resp == 1:
		session_control.ShowOptions()
	box.destroy()
